#include "Alignment.h"
#include <fstream>
#include <cstdlib>

using namespace std;

Alignment::Alignment( Network net1, Network net2)
{
	if( net1.size > net2.size )
	{
		reverse = true;
		network1 = net2;
		network2 = net1;
	}
	else
	{
		reverse = false;
		network1 = net1;
		network2 = net2;
	}

	if(network1.maxDeg > network2.maxDeg)
		maxDeg = network1.maxDeg;
	else
		maxDeg = network2.maxDeg;

	similarity = new float*[network1.size];

	for(int i=0; i<network1.size; i++)
	{
		similarity[i] = new float[network2.size];
	}

}

float Alignment::computeSimilarity(int node1, int node2)
{
	bool *al;
	float *sim;
	int last;

	float tempS = 0;

	int deg1 = network1.deg[node1];
	int deg2 = network2.deg[node2];

	if( network1.self[ node1 ] )
		deg1 += 2;
	if( network2.self[ node2 ] )
		deg2 += 2;

	if (network1.deg[node1] <= network2.deg[node2])
	{
		al = new bool[ network2.deg[node2] ];
		sim = new float[ network1.deg[node1] ];
		for(int i=0; i<network2.deg[node2]; i++)
		{
			al[i] = false;
		}
		for(int i=0; i<network1.deg[node1]; i++)
		{
			sim[i] = 0;
		}
		for(int i=0; i<network1.deg[node1]; i++)
		{
			for(int j=0; j<network2.deg[node2]; j++)
			{
				if( !al[j] && sim[i] < similarity[ network1.neighbor[node1][i] ][ network2.neighbor[node2][j] ] )
				{
					sim[i] = similarity[ network1.neighbor[node1][i] ][ network2.neighbor[node2][j] ];
					last = j;
				}
			}
			al[last] = true;
			tempS += sim[i];
		}
	}
	else
	{
		al = new bool[network1.deg[node1]];
		sim = new float[ network2.deg[node2] ];
		for(int i=0; i<network1.deg[node1]; i++)
		{
			al[i] = false;
		}
		for(int i=0; i<network2.deg[node2]; i++)
		{
			sim[i] = 0;
		}
		for(int i=0; i<network2.deg[node2]; i++)
		{
			for(int j=0; j<network1.deg[node1]; j++)
			{
				if( !al[j] && sim[i] < similarity[ network1.neighbor[node1][j] ][ network2.neighbor[node2][i] ])
				{
					sim[i] = similarity[ network1.neighbor[node1][j] ][ network2.neighbor[node2][i] ];
					last = j;
				}
			}
			al[last] = true;
			tempS += sim[i];
		}
	}

	if( network1.self[ node1 ] && network2.self[ node2 ] )
			tempS += 2;;

		if( deg2 >= deg1 && deg2 > 0 )
			tempS /= deg2;
		else if ( deg1 > deg2 && deg1 > 0)
			tempS /= deg1;
		else
			tempS = 1.0;


	delete [] al;
	delete [] sim;
	return tempS;
}

void Alignment::setSimilarities(int iteration, double bb, double cc)
{
	float ** tempSim;

	tempSim = new float*[network1.size];

	for(int i=0; i<network1.size; i++)
	{
		tempSim[i] = new float[network2.size];
	}

	for(int i=0; i<network1.size; i++)
		for(int j=0; j<network2.size; j++)
		{
			similarity[i][j] = 1.0;
		}


	if( bb > 0 )
	{

		float *sim1 = new float[network1.size];
		float *sim2 = new float[network2.size];

		for(int i=0; i<network1.size; i++)
			sim1[ i ] = 0;
		for(int i=0; i<network2.size; i++)
			sim2[ i ] = 0;



		string line;
		string token;
		int id1, id2;



		string f1,f2,f3;

		f1 = network1.name;
		f1.append("-");
		f1.append(network1.name);
		f1.append(".val");
		ifstream file1(f1.c_str());

		while (getline(file1, line))
		{

			istringstream tokenizer(line);
			string token;

			getline(tokenizer, token, '\t');

			id1 = network1.getID( token );

			getline(tokenizer, token, '\t');

			if(token.at(token.length()-1)=='\n')
				token = token.substr(0,token.length()-1);

			id2 =  network1.getID( token );

			if ( id1 == id2 )
			{
				getline(tokenizer, token, '\t');

				if(token.at(token.length()-1)=='\n')
					token = token.substr(0,token.length()-1);

				sim1[ id1 ] = (float)atof(token.c_str());
			}

		}




		f2 = network2.name;
		f2.append("-");
		f2.append(network2.name);
		f2.append(".val");

		ifstream file2(f2.c_str());

		while (getline(file2, line))
		{

			istringstream tokenizer(line);
			string token;

			getline(tokenizer, token, '\t');

			id1 = network1.getID( token );

			getline(tokenizer, token, '\t');

			if(token.at(token.length()-1)=='\n')
				token = token.substr(0,token.length()-1);

			id2 =  network1.getID( token );

			if ( id1 == id2 )
			{
				getline(tokenizer, token, '\t');

				if(token.at(token.length()-1)=='\n')
					token = token.substr(0,token.length()-1);

				sim2[ id1 ] = (float)atof(token.c_str());
			}

		}

		if( reverse )
		{
			f3 = network2.name;
			f3.append("-");
			f3.append(network1.name);
			f3.append(".val");
		}
		else
		{
			f3 = network1.name;
			f3.append("-");
			f3.append(network2.name);
			f3.append(".val");
		}

		ifstream file3(f3.c_str());

		while (getline(file3, line))
		{

			istringstream tokenizer(line);
			string token;

			getline(tokenizer, token, '\t');

			id1 = network1.getID( token );

			getline(tokenizer, token, '\t');

			if(token.at(token.length()-1)=='\n')
				token = token.substr(0,token.length()-1);

			id2 =  network1.getID( token );

			getline(tokenizer, token, '\t');

			if(token.at(token.length()-1)=='\n')
				token = token.substr(0,token.length()-1);

			similarity[ id1 ][ id2 ] = (float)( ( 1 - bb )* 1.0 + bb * 2 * atof(token.c_str()) / ( sim1[ id1 ] + sim2[ id2 ] ) );
		}
	}

	int maxSim1;
	int maxSim2;
	int minSim1;
	int minSim2;
	double sumSim;

	ofstream simFile( "simLog.txt");

//	cout << "Set Similarities:" << endl;
	for(int it=0; it<iteration; it++)
	{

		maxSim1 = 0;
		maxSim2 = 0;
		minSim1 = 0;
		minSim2 = 0;
		sumSim = 0;


//		cout << "--> " << it << endl;
		for(int i=0; i<network1.size; i++)
		{
			if( i % 1000 == 0)
//				cout << i << endl;
			for(int j=0; j<network2.size; j++)
			{
				tempSim[i][j] = (float)( ( 1 - cc ) * similarity[i][j] + cc * computeSimilarity( i, j ) );
			}
			//cout << i << endl;
		}
		for(int i=0; i<network1.size; i++)
		{
			for(int j=0; j<network2.size; j++)
			{
				similarity[i][j] = tempSim[i][j];
				sumSim += similarity[i][j] / network2.size;
				if( similarity[i][j] > similarity[ maxSim1 ][ maxSim2 ] )
				{
					maxSim1 = i;
					maxSim2 = j;
				}
				else if( similarity[i][j] < similarity[ minSim1 ][ minSim2 ] )
				{
					minSim1 = i;
					minSim2 = j;
				}
			}
		}
		sumSim /= network1.size;

		simFile << it << " --------------------------------------" << endl;
		simFile << "Average : " << ( sumSim / network1.size ) << endl;
		simFile << "Maximum : " << " deg[" << maxSim1 << "]=" << network1.deg[maxSim1] << ", deg[" << maxSim2 << "]=" << network2.deg[maxSim2] << ", similarity[" << maxSim1 << "][" << maxSim2 << "]=" << similarity[maxSim1][maxSim2] << endl;
		simFile << "Minimum : " << " deg[" << minSim1 << "]=" << network1.deg[minSim1] << ", deg[" << minSim2 << "]=" << network2.deg[minSim2] << ", similarity[" << minSim1 << "][" << minSim2 << "]=" << similarity[minSim1][minSim2] << endl;

	}

}

void Alignment::align(double aa)
{

	float **score;

	score = new float*[network1.size];

	for(int i=0; i<network1.size; i++)
	{
		score[i]= new float[network2.size];
	}
	float ss;
	//float epsil = aa / network1.size;
	float epsil = 0;

	for(int i=0; i<network1.size; i++)
		for(int j=0; j<network2.size; j++)
		{
			score[i][j]= 0;
		}
	float *tempScore1;
	float *tempScore2;

	tempScore1 = new float[network1.size];
	for(int i=0; i<network1.size; i++)
	{
		tempScore1[i] = 0;
		for(int j=0; j<network1.deg[i]; j++)
			tempScore1[i] += (float)( 1.0 / network1.deg[network1.neighbor[i][j]] );
	}
	tempScore2 = new float[network2.size];
	for(int i=0; i<network2.size; i++)
	{
		tempScore2[i] = 0;
		for(int j=0; j<network2.deg[i]; j++)
			tempScore2[i] += (float)( 1.0 / network2.deg[network2.neighbor[i][j]] );
	}
	ofstream detailFile( "alignmentDetails.txt");

	for(int i=0; i<network1.size; i++)
		for(int j=0; j<network2.size; j++)
		{
			if( network1.self[ i ] && network2.self[ j ])
			{
				score[i][j] = (tempScore1[i]<tempScore2[j])?(tempScore1[i]+1)/maxDeg:(tempScore2[j]+1)/maxDeg;
			}
			else
				score[i][j] = (tempScore1[i]<tempScore2[j])?tempScore1[i]/maxDeg:tempScore2[j]/maxDeg;
                ss = score[i][j];
                score[i][j] = (float)( (1 - aa) * ss +  aa * similarity[i][j] );
                if( score[i][j] < 0 )
                    detailFile << "--------------------------  " << i << " : " << network1.deg[i] << " ----> " << j << " : " << network2.deg[j] << "    Score : " << score[i][j] << endl;

		}
//	cout << "score finished ! " << endl;


	int nodeSize = network1.size;
	alignment = new int[nodeSize];
	int *nodeScore = new int[nodeSize];
	bool *aligned1 = new bool[nodeSize];
	float *minus = new float[nodeSize];
	bool *aligned2 = new bool[network2.size];
	float *negScore1 = new float[nodeSize];
	float *negScore2 = new float[network2.size];
	int **posScore;

	posScore = new int*[network1.size];

	for(int i=0; i<network1.size; i++)
	{
		posScore[i]= new int[network2.size];
	}



	for(int i=0; i<nodeSize; i++)
	{
		nodeScore[i] = 0;
		aligned1[i] = false;
		negScore1[i] = 0;
	}

	for(int i=0; i<network2.size; i++)
	{
		aligned2[i] = false;
		negScore2[i] = 0;
	}

	for(int i=0; i<nodeSize; i++)
		for(int j=0;j<network2.size; j++)
		{
			posScore[i][j]=0;
			if(score[i][j]>score[i][nodeScore[i]])
				nodeScore[i] = j;
		}
	int maxScore;

	bool ast = true;

//	cout << "Alignment:"<< endl;
	for(int i=0; i<nodeSize; i++)
	{
		maxScore = -1;
		int cnt = 0;
		for(int j=0; j<nodeSize; j++)
		{
			if(!aligned1[j])
			{
				if(maxScore < 0 )
					maxScore = j;
				else if(score[j][nodeScore[j]] > score[maxScore][nodeScore[maxScore]])
				{
					maxScore = j;
				}
				cnt++;
			}
		}
		if( i+ cnt != 5626 && ast )
		{

				ast = false;
		}

	//	cout << maxScore << " --> " << nodeScore[maxScore] << endl;

		alignment[maxScore] = nodeScore[maxScore];
		aligned1[maxScore] = true;
		aligned2[nodeScore[maxScore]] = true;
		aa -= epsil;

		detailFile << maxScore << " : " << network1.deg[maxScore] << " ----> " << nodeScore[maxScore] << " : " << network2.deg[nodeScore[maxScore]] << "    Score : " << score[maxScore][nodeScore[maxScore]] << "     Similarity" << aa * similarity[maxScore][nodeScore[maxScore]] << endl;

		for(int j=0; j<network1.deg[maxScore]; j++)
			negScore1[network1.neighbor[maxScore][j]] += (float)( 1.0 / network1.deg[maxScore] );

		for(int k=0; k<network2.deg[nodeScore[maxScore]]; k++)
			negScore2[network2.neighbor[nodeScore[maxScore]][k]] += (float)( 1.0 / network2.deg[ nodeScore[maxScore] ] );


		for(int j=0; j<network1.deg[maxScore]; j++)
			for(int k=0; k<network2.deg[nodeScore[maxScore]]; k++)
				if( !aligned1[network1.neighbor[maxScore][j]] && !aligned2[network2.neighbor[nodeScore[maxScore]][k]])
				{
					if(network1.deg[network1.neighbor[maxScore][j]]==1 && network2.deg[network2.neighbor[nodeScore[maxScore]][k]]==1)
					{
						alignment[network1.neighbor[maxScore][j]] = network2.neighbor[nodeScore[maxScore]][k];

						//cout << network1.neighbor[maxScore][j] << " --> " << network2.neighbor[nodeScore[maxScore]][k] << endl;

						aligned1[network1.neighbor[maxScore][j]] = true;
						aligned2[network2.neighbor[nodeScore[maxScore]][k]] = true;

						detailFile << "$ " << network1.neighbor[maxScore][j] << " : " << network1.deg[network1.neighbor[maxScore][j]] << " ----> " << network2.neighbor[nodeScore[maxScore]][k] << " : " << network2.deg[network2.neighbor[nodeScore[maxScore]][k]] << "    Score : " << score[network1.neighbor[maxScore][j]][network2.neighbor[nodeScore[maxScore]][k]] << endl;

						aa -= epsil;
						i++;
					}
				}

		for(int j=0; j<network1.deg[maxScore]; j++)
			for(int k=0; k<network2.deg[nodeScore[maxScore]]; k++)
					posScore[network1.neighbor[maxScore][j]][network2.neighbor[nodeScore[maxScore]][k]]++;



		for(int j=0; j<network1.deg[maxScore]; j++)
			if( !aligned1[network1.neighbor[maxScore][j]] )
				for( int k=0; k<network2.size; k++ )
				{
					if( network1.self[ network1.neighbor[maxScore][j] ] && network2.self[ k ])
						score[ network1.neighbor[maxScore][j] ][k] = ( ( tempScore1[ network1.neighbor[maxScore][j] ] - negScore1[ network1.neighbor[maxScore][j] ] ) < ( tempScore2[k] - negScore2[k] ) )?( tempScore1[ network1.neighbor[maxScore][j] ] - negScore1[ network1.neighbor[maxScore][j] ]  + 1 )/maxDeg : ( tempScore2[k] - negScore2[k] + 1)/maxDeg;
					else
						score[ network1.neighbor[maxScore][j] ][k] = ( ( tempScore1[ network1.neighbor[maxScore][j] ] - negScore1[ network1.neighbor[maxScore][j] ] ) < ( tempScore2[k] - negScore2[k] ) )?( tempScore1[ network1.neighbor[maxScore][j] ] - negScore1[ network1.neighbor[maxScore][j] ] )/maxDeg : ( tempScore2[k] - negScore2[k] )/maxDeg;
					ss =  ((float)posScore[ network1.neighbor[maxScore][j] ][k])/maxDeg + score[ network1.neighbor[maxScore][j] ][k];
					score[ network1.neighbor[maxScore][j] ][k] =  (float)( (1 - aa) * ss + aa * similarity[ network1.neighbor[maxScore][j] ][k] );

				}

		for(int k=0; k<network2.deg[nodeScore[maxScore]]; k++)
			if( !aligned2[network2.neighbor[nodeScore[maxScore]][k]] )
				for( int j=0; j<network1.size; j++ )
				{
					if( network1.self[ j ] && network2.self[ network2.neighbor[nodeScore[maxScore]][k] ])
						score[j][ network2.neighbor[nodeScore[maxScore]][k] ] = ( ( tempScore1[j] - negScore1[j] ) < ( tempScore2[ network2.neighbor[nodeScore[maxScore]][k] ] - negScore2[ network2.neighbor[nodeScore[maxScore]][k] ] ) )?( tempScore1[j] - negScore1[j] + 1 )/maxDeg : ( tempScore2[ network2.neighbor[nodeScore[maxScore]][k] ] - negScore2[ network2.neighbor[nodeScore[maxScore]][k] ] + 1 )/maxDeg;
					else
						score[j][ network2.neighbor[nodeScore[maxScore]][k] ] = ( ( tempScore1[j] - negScore1[j] ) < ( tempScore2[ network2.neighbor[nodeScore[maxScore]][k] ] - negScore2[ network2.neighbor[nodeScore[maxScore]][k] ] ) )?( tempScore1[j] - negScore1[j] )/maxDeg : ( tempScore2[ network2.neighbor[nodeScore[maxScore]][k] ] - negScore2[ network2.neighbor[nodeScore[maxScore]][k] ])/maxDeg;
					ss = ((float)posScore[j][ network2.neighbor[nodeScore[maxScore]][k] ])/maxDeg + score[j][ network2.neighbor[nodeScore[maxScore]][k] ];
					score[j][ network2.neighbor[nodeScore[maxScore]][k] ] = (float)( (1 - aa) * ss + aa * similarity[j][ network2.neighbor[nodeScore[maxScore]][k] ] );
				}


		for(int k=0; k<network2.deg[nodeScore[maxScore]]; k++)
			if( !aligned2[network2.neighbor[nodeScore[maxScore]][k]] )
				for( int j=0; j<network1.size; j++ )
				{
					if( nodeScore[ j ] == network2.neighbor[nodeScore[maxScore]][k] )
					{
							for(int l=0;l<network2.size; l++)
								if(score[ j ][l]>score[j][nodeScore[j]])
									nodeScore[j] = l;
					}
				}

		for(int j=0; j<network1.deg[maxScore]; j++)
			if( !aligned1[network1.neighbor[maxScore][j]] )
				for( int k=0; k<network2.size; k++ )
				{
					if(score[ network1.neighbor[maxScore][j] ][k]>score[ network1.neighbor[maxScore][j] ][nodeScore[ network1.neighbor[maxScore][j] ]])
									nodeScore[ network1.neighbor[maxScore][j] ] = k;
				}

		for(int j=0; j<nodeSize; j++)
		{
			if(aligned2[nodeScore[j]])
			{
				score[j][nodeScore[j]] = -1;
			}
		}

		for(int j=0; j<nodeSize; j++)
		{
			if(aligned2[nodeScore[j]])
			{
				for(int k=0;k<network2.size; k++)
					if(score[j][k]>score[j][nodeScore[j]] && !aligned2[k])
						nodeScore[j] = k;
			}
		}

	}


	evaluate();

}


void Alignment::improve(double aa)
{
	int *align = new int[network2.size];
	int *align2 = new int[network2.size];
	int *alScore = new int[network2.size];
	int totalScore;
	int minEdge = ( network1.numOfEdge > network2.numOfEdge)? network2.numOfEdge : network1.numOfEdge;

	for( int i = 0; i < network2.size; i++)
	{
		align[ i ] = -1;
		align2[ i ] = -1;
		alScore[ i ] = 0;
	}
	for( int i = 0; i < network1.size; i++)
	{
		align[ i ] = alignment[ i ];
		align2[ alignment[ i ] ] = i;
	}

	totalScore = 0;
	for(int i=0; i<network1.size; i++)
	{
		if( network1.self[ i ] && network2.self[ align[ i ] ])
		{
			alScore[i] ++;
			totalScore += 2;
		}

		for(int j=0; j<network1.deg[i]; j++)
			for( int l=0; l < network2.deg[align[i]]; l++ )
				if(network2.neighbor[ align[i] ][l] == align[ network1.neighbor[i][j] ])
				{
					alScore[i]++;
					totalScore++;
				}
	}


	for( int i = 0,j = network1.size; i < network2.size; i++)
		if( align2[ i ] < 0 )
		{
			align[ j ] = i;
			align2[ i ] = j;
			j++;
		}

	int ts;

	int mts, mi, mj,tmp;

	for( int counter=0;;counter++)
	{
		mts = -100;
		mi = 0;
		mj = 0;
		for( int i = 0; i < network1.size; i++)
		{
			for( int j=0,p = 1; p < (network1.size + 1)/2 ; p++)
			{

				j = ( i + p )% network1.size;

			//	cout << "1" << endl;
				ts = 0;
				if( network1.self[ i ] && network2.self[ align[ j ] ])
					ts++;

			//	cout << "2" << endl;
				if( network1.self[ j ] && network2.self[ align[ i ] ])
					ts++;

			//	cout << "3" << endl;
				for(int k=0; k<network1.deg[i]; k++)
				{
					if( network1.neighbor[i][k] == j )
					{
						for( int l=0; l < network2.deg[ align[j] ]; l++ )
							if( network2.neighbor[ align[j] ][l] == align[i] )
								ts++;
					}
					else
					{
						for( int l=0; l < network2.deg[ align[j] ]; l++ )
							if(network2.neighbor[ align[j] ][l] == align[ network1.neighbor[i][k] ])
								ts++;
					}
				}
			//	cout << "4" << endl;
				for(int k=0; k<network1.deg[j]; k++)
				{
					if( network1.neighbor[j][k] == i )
					{
						for( int l=0; l < network2.deg[ align[i] ]; l++ )
							if( network2.neighbor[ align[i] ][l] == align[j] )
								ts++;
					}
					else
					{
						for( int l=0; l < network2.deg[ align[i] ]; l++ )
							if( network2.neighbor[ align[i] ][l] == align[ network1.neighbor[j][k] ])
								ts++;
					}
				}
			//	cout << "5" << endl;
				ts = ts - alScore[i] - alScore[j];


				if(ts > mts)
				{
					mi = i;
					mj = j;
					mts = ts;
				}
			}


			for( int j=network1.size; j < network2.size ; j++)
			{

			//	cout << "1" << endl;
				ts = 0;
				if( network1.self[ i ] && network2.self[ align[ j ] ])
					ts++;

			//	cout << "3" << endl;
				for(int k=0; k<network1.deg[i]; k++)
					for( int l=0; l < network2.deg[ align[j] ]; l++ )
						if(network2.neighbor[ align[j] ][l] == align[ network1.neighbor[i][k] ])
							ts++;
			//	cout << "4" << endl;
			//	cout << "5" << endl;
				ts = ts - alScore[i] - alScore[j];


				if(ts > mts)
				{
					mi = i;
					mj = j;
					mts = ts;
				}
			}
		}
		//cout << "6" << endl;
		tmp = align[ mi ];
		align[ mi ] = align[ mj ];
		align[ mj ] = tmp;
		//		cout << "7" << endl;
		totalScore = 0;
		for(int i=0; i<network1.size; i++)
		{//				cout << "8" << endl;
			alScore[i] = 0;
			if( network1.self[ i ] && network2.self[ align[ i ] ])
			{
				alScore[i] ++;
				totalScore += 2;
			}
		//					cout << "9" << endl;
			for(int j=0; j<network1.deg[i]; j++)
				for( int l=0; l < network2.deg[align[i]]; l++ )
					if(network2.neighbor[ align[i] ][l] == align[ network1.neighbor[i][j] ])
					{
						alScore[i]++;
						totalScore++;
					}
		}

	//	cout << "################    " << counter << "  -->  " << ((float)totalScore)/(2*minEdge) << endl;;
	}
}



void Alignment::SAalign(double aa, int n)
{

	cout << "start" << endl;
	int it = 10000;
	int **align1 = new int*[n];
	int **align2 = new int*[n];
	int **nAlign1 = new int*[n];
	int **nAlign2 = new int*[n];
	int *aScore = new int[n];
	int *mut1 = new int[n];
	int *mut2 = new int[n];
	int *cross = new int[n];
	int *partner = new int[n];
	for( int i = 0; i < n; i++)
	{
		align1[i] = new int[network1.size];
		align2[i] = new int[network2.size];
		nAlign1[i] = new int[network1.size];
		nAlign2[i] = new int[network2.size];
	}


	cout << align2[1][2821] << endl;
	int best;
	int start;
	int kk;
	int sc;
	int minEdge = ( network1.numOfEdge > network2.numOfEdge)? network2.numOfEdge : network1.numOfEdge;

	cout << "Definition finished " << endl;


	srand ( (unsigned)time(0) );

	for( int i = 0; i < n; i++)
	{
		for( int j = 0; j < network1.size; j++)
			align1[i][j] = -1;
		for( int j = 0; j < network2.size; j++)
			align2[i][j] = -1;

		if(i<0)
		{
			for( int j = 0; j < network1.size; j++)
			{
				align1[i][j] = alignment[j];
				align2[i][alignment[j]] = j;
			}
		}
		else
		{
			start = rand() % network1.size;

			for( int j = start; j < network1.size+start; j++)
			{
				kk = j % network1.size;

				best = -1;
				for(int l=0; l<network2.size; l++)
					if( align2[i][l] <0 && ( best < 0 || similarity[kk][l] > similarity[kk][best]))
						best = l;

				align1[i][kk] = best;
				align2[i][best] = kk;
			}

		}

		aScore[i]=0;


		for(int k=0; k<network1.size; k++)
		{

			if( network1.self[ k ] && network2.self[ align1[i][ k ] ])
				aScore[i] += 2;
			for(int l=0; l<network1.deg[k]; l++)
			{
				for( int m=0; m < network2.deg[align1[i][k]]; m++ )
				{
					if(network2.neighbor[ align1[i][k] ][m] == align1[i][ network1.neighbor[k][l] ])
					{
						aScore[i]++;
					}
				}
			}
		}
		aScore[i] = aScore[i]/2;

	}


	cout << "First finished!" << endl;

	int bes;

	int a,b,c,d;

	int totalScore;

	for(int i=0; i<it; i++)
	{
		for(int j=0; j<n; j++)
		{
			bes = 0;

			partner[j] = rand() % n;
			cross[j] = rand()%network1.size;
			mut1[j] = rand()%network1.size;
			mut2[j] = rand()%network2.size;


			for( int k = 0; k < network2.size; k++)
				nAlign2[j][k] = -1;
			for( int k = 0; k < network1.size; k++ )
			{
				if( k < cross[j])
				{
					nAlign1[j][k] = align1[j][k];
					nAlign2[j][ nAlign1[j][k] ] = align2[j][align1[j][k]];
				}
				else
				{
					a = k;
					b = align1[j][k];
					c = align2[j][ align1[ partner[j] ][k] ];
					d = align1[ partner[j] ][k];

					nAlign1[j][ a ] = d;
					nAlign2[j][ b ] = c;
					if( c > 0 )
						nAlign1[j][ c ] = b;
					nAlign2[j][d] = a;
				}
			}

			a = mut1[j];
			b = align1[j][a];
			c = align2[j][mut2[j]];
			d = mut2[j];


			nAlign1[j][ a ] = d;
			nAlign2[j][ b ] = c;
			if( c > 0 )
				nAlign1[j][ c ] = b;
			nAlign2[j][d] = a;


			totalScore=0;
			for(int k=0; k<network1.size; k++)
			{
				if( network1.self[ k ] && network2.self[j][ nAlign1[ k ] ])
					totalScore += 2;

				for(int l=0; l<network1.deg[k]; l++)
					for( int m=0; m < network2.deg[nAlign1[j][k]]; m++ )
						if(network2.neighbor[ nAlign1[j][k] ][m] == nAlign1[j][ network1.neighbor[k][l] ])
							totalScore++;
			}
			totalScore = totalScore/2;

			if(totalScore > aScore[j])
			{
				bes = 1;
				aScore[j] = totalScore;
			}


			for( int k = 0; k < network2.size; k++)
					nAlign2[j][k] = -1;
			for( int k = network1.size-1; k >=0; k-- )
			{
				if( k >= cross[j])
				{
					nAlign1[j][k] = align1[j][k];
					nAlign2[j][ nAlign1[j][k] ] = align2[j][align1[j][k]];
				}
				else
				{
					a = k;
					b = align1[j][k];
					c = align2[j][ align1[ partner[j] ][k] ];
					d = align1[ partner[j] ][k];

					nAlign1[j][ a ] = d;
					nAlign2[j][ b ] = c;
					if( c > 0 )
						nAlign1[j][ c ] = b;
					nAlign2[j][d] = a;
				}
			}

			a = mut1[j];
			b = align1[j][a];
			c = align2[j][mut2[j]];
			d = mut2[j];


			nAlign1[j][ a ] = d;
			nAlign2[j][ b ] = c;
			if( c > 0 )
				nAlign1[j][ c ] = b;
			nAlign2[j][d] = a;

			totalScore=0;
			for(int k=0; k<network1.size; k++)
			{
				if( network1.self[ k ] && network2.self[ nAlign1[j][ k ] ])
					totalScore += 2;

				for(int l=0; l<network1.deg[k]; l++)
					for( int m=0; m < network2.deg[nAlign1[j][k]]; m++ )
						if(network2.neighbor[ nAlign1[j][k] ][m] == nAlign1[j][ network1.neighbor[k][l] ])
							totalScore++;
			}
			totalScore = totalScore/2;

			if(totalScore > aScore[j])
			{
				bes = 2;
				aScore[j] = totalScore;
			}
			else if ( bes ==1)
			{

				for( int k = 0; k < network2.size; k++)
					nAlign2[j][k] = -1;
				for( int k = 0; k < network1.size; k++ )
				{
					if( k < cross[j])
					{
						nAlign1[j][k] = align1[j][k];
						nAlign2[j][ nAlign1[j][k] ] = align2[j][align1[j][k]];
					}
					else
					{
						a = k;
						b = align1[j][k];
						c = align2[j][ align1[ partner[j] ][k] ];
						d = align1[ partner[j] ][k];

						nAlign1[j][ a ] = d;
						nAlign2[j][ b ] = c;
						if( c > 0 )
							nAlign1[j][ c ] = b;
						nAlign2[j][d] = a;
					}
				}

				a = mut1[j];
				b = align1[j][a];
				c = align2[j][mut2[j]];
				d = mut2[j];


				nAlign1[j][ a ] = d;
				nAlign2[j][ b ] = c;
				if( c > 0 )
					nAlign1[j][ c ] = b;
				nAlign2[j][d] = a;
			}
		}

		for( int j=0; j < n; j++)
		{
			for( int k=0; k < network1.size; k++)
				align1[j][k] = nAlign1[j][k];
			for( int k=0; k < network2.size; k++)
				align2[j][k] = nAlign2[j][k];
		}

	sc = 0;

	for( int j = 0; j < n; j++)
		if( aScore[sc] < aScore[j] )
			sc = j;



	cout << i << " :   Score= " << ((float)aScore[sc])/minEdge << endl;;

	}


	for( int i=0; i<network1.size; i++)
	{
		alignment[ i ] = align1[ sc ][ i ];
	}

}


void Alignment::evaluate(void)
{
	CCCV = getCCCV();
	CCCE = getCCCE();
	EC = getEC();
	NC = getNC();
	IC = getIC();
}

int Alignment::getCCCV(void)
{
	int compNum = 1;
	int *q = new int[network1.size];
	comp = new int[network1.size];
	for(int i=0; i<network1.size; i++)
	{
		comp[i] = network1.size;
		q[i] = i;
	}

	int last = 0;
	int tmp = 0;
	for(int i=0; i<network1.size; i++)
	{
		if(comp[i]==network1.size)
		{
			q[0] = i;
			comp[i] = compNum;
			compNum++;
			last = 1;
			for(int k=0; k<last; k++)
				for(int j=0; j<network1.deg[q[k]]; j++)
					if( comp[q[k]] < comp[network1.neighbor[q[k]][j]])
					{
						for( int l=0; l < network2.deg[alignment[q[k]]]; l++ )
							if(network2.neighbor[alignment[q[k]]][l] == alignment[network1.neighbor[q[k]][j]])
							{
								comp[network1.neighbor[q[k]][j]] = comp[q[k]];
								q[last] = network1.neighbor[q[k]][j];
								last++;
							}
					}
		}
	}

	int *subGraph;

	subGraph = new int[compNum-1];
	for(int i=0; i<compNum-1; i++)
		subGraph[i] = 0;
	for(int i=0; i<network1.size; i++)
		subGraph[comp[i]-1]++;

	maxComp = 0;
	for(int i=0; i<compNum-1; i++)
	{
		if(subGraph[maxComp] < subGraph[i])
			maxComp = i;
	}

	return subGraph[maxComp];
}


int Alignment::getCCCE(void)
{
	int edgeComp = 0;

	for(int i=0; i<network1.size; i++)
	{

		if( network1.self[ i ] && network2.self[ alignment[ i ] ] && comp[i]-1 == maxComp)
			edgeComp += 2;

		for(int j=0; j<network1.deg[i]; j++)
			for( int l=0; l < network2.deg[alignment[i]]; l++ )
				if(network2.neighbor[ alignment[i] ][l] == alignment[network1.neighbor[i][j]])
					if( comp[i]-1 == maxComp)
						edgeComp++;
	}

	return ( edgeComp / 2 );
}

float Alignment::getEC(void)
{
	int totalScore=0;

	for(int i=0; i<network1.size; i++)
	{

		if( network1.self[ i ] && network2.self[ alignment[ i ] ])
			totalScore += 2;

		for(int j=0; j<network1.deg[i]; j++)
			for( int l=0; l < network2.deg[alignment[i]]; l++ )
				if(network2.neighbor[ alignment[i] ][l] == alignment[ network1.neighbor[i][j] ])
					totalScore++;
	}

	int minEdge = ( network1.numOfEdge > network2.numOfEdge)? network2.numOfEdge : network1.numOfEdge;

	return ( (float) totalScore ) / ( 2 * minEdge );
}


float Alignment::getNC(void)
{
	int nc = 0;

	for( int i=0; i<network1.size; i++)
		if( network1.getID( network2.getName ( alignment[i] ) ) == i )
			nc++;

	return ( (float)nc / network1.size );
}

float Alignment::getIC(void)
{
	int ic = 0;
	for(int i=0; i<network1.size; i++)
	{

		if( network1.self[ i ] && network2.self[ alignment[ i ] ] && alignment[i] == i )
			ic += 2;

		for(int j=0; j<network1.deg[i]; j++)
			for( int l=0; l < network2.deg[alignment[i]]; l++ )
				if(network2.neighbor[ alignment[i] ][l] == alignment[network1.neighbor[i][j]])
					if( network1.getID( network2.getName ( alignment[i] ) ) == i && network1.getID( network2.getName ( alignment[network1.neighbor[i][j]] ) )== network1.neighbor[i][j] )
						ic++;
	}

	int minEdge = ( network1.numOfEdge > network2.numOfEdge)? network2.numOfEdge : network1.numOfEdge;

	return ( (float) ic ) / ( 2 * minEdge );
}

void Alignment::outputEvaluation(string name)
{


	cout << "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
	cout << "*** CONNECTED COMPONENTS SIZE : " << endl;
	cout << "Nodes = " << CCCV << endl;
	cout << "Edges = " << CCCE << endl;

	cout << "===============================================================" << endl;
	if(reverse)
	{
		cout << "G1:  Nodes : " << network2.size << "  - Edges : " << network2.numOfEdge << endl;
		cout << "G2:  Nodes : " << network1.size << "  - Edges : " << network1.numOfEdge << endl;
	}
	else
	{
		cout << "G1:  Nodes : " << network1.size << "  - Edges : " << network1.numOfEdge << endl;
		cout << "G2:  Nodes : " << network2.size << "  - Edges : " << network2.numOfEdge << endl;
	}

	cout << "EC : " << EC << endl;
	cout << "NC : " << NC << endl;
	cout << "IC : " << IC << endl;



	string outFile = name;
	outFile.append(".eval");
	ofstream outputFile( outFile.c_str());



	outputFile << "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
	outputFile << "*** CONNECTED COMPONENTS SIZE : " << endl;
	outputFile << "Nodes = " << CCCV << endl;
	outputFile << "Edges = " << CCCE << endl;

	outputFile << "===============================================================" << endl;
	if(reverse)
	{
		outputFile << "G1:  Nodes : " << network2.size << "  - Edges : " << network2.numOfEdge << endl;
		outputFile << "G2:  Nodes : " << network1.size << "  - Edges : " << network1.numOfEdge << endl;
	}
	else
	{
		outputFile << "G1:  Nodes : " << network1.size << "  - Edges : " << network1.numOfEdge << endl;
		outputFile << "G2:  Nodes : " << network2.size << "  - Edges : " << network2.numOfEdge << endl;
	}

	outputFile << "EC : " << EC << endl;
	outputFile << "NC : " << NC << endl;
	outputFile << "IC : " << IC << endl;


}

void Alignment::outputAlignment(string name)
{

	string alignFile = name;

	alignFile.append(".alignment");


	ofstream alignmentFile( alignFile.c_str());

	if(reverse)
		for(int i=0; i<network1.size; i++)
			alignmentFile << network1.getName( alignment[ i ] ) << ' ' << network2.getName( i )<< endl;
	else
		for(int i=0; i<network1.size; i++)
			alignmentFile << network1.getName( i ) << ' ' << network2.getName( alignment[ i ] )<< endl;
}

Alignment::Alignment(void)
{
}

Alignment::~Alignment(void)
{
}
